from cardlink.types import Bill
from cardlink.methods.base import CardLinkBaseMethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    import cardlink


class toggleActivity:
    class toggleActivityMethod(CardLinkBaseMethod):
        id: str
        active: bool

        __return_type__ = Bill
        __api_method__ = "bill/toggle_activity"
        __request_type__ = "POST"

    async def toggle_activity(
            self: "cardlink.CardLink",
            id: str,
            active: bool
    ) -> Bill:
        """
        Активировать/деактивировать счет на оплату
        Деактивация счета может быть полезна для ручного отключения платежной ссылки с типом MULTI, т.к. такой тип платежных ссылок не меняет свой статус после оплаты и может быть оплачен еще сколько угодно раз.

        :param id: Уникальный идентификатор счета
        :param active: False - деактивировать счет, True - активировать счет

        :return: :class:`Bill` object
        """
        params = locals()

        params["active"] = int(params["active"])

        return await self(self.toggleActivityMethod(**params))



import datetime
from pydantic import field_validator
from cardlink.types.base import BaseCardLinkTypes
from cardlink.types.PayoutStatus import PayoutStatus
from cardlink.types.Currency import Currency


class Payout(BaseCardLinkTypes):
    """
    Payout object

    Source: https://cardlink.link/merchant/api#payout-resource
    """

    id: str | None = None
    """Уникальный идентификатор выплаты"""
    status: PayoutStatus | str | None = None
    """Статус выплаты"""
    order_id: str | None = None
    """Уникальный идентификатор заказа"""
    account_identifier: str | None = None
    """Платежный аккаунт, на который производится выплата"""
    amount: int | float | str | None = None
    """В случае recipient_pays_commission:false поле amount - сумма выплаты с учетом комиссии, в случае recipient_pays_commission:true поле amount - оригинальная сумма выплаты"""
    account_amount: int | float | str | None = None
    """Сумма, списанная с баланса"""
    commission: int | float | str | None = None
    """Комиссия сервиса"""
    account_commission: int | float | str | None = None
    """Комиссия сервиса в валюте баланса"""
    currency: Currency | str | None = None
    """Валюта выплаты"""
    account_currency: Currency | str | None = None
    """Валюта баланса"""
    created_at: datetime.datetime | None = None
    """Дата и время создания выплаты"""

    @field_validator("created_at", mode="before")
    @classmethod
    def parse_datetime(cls, v):
        """Преобразование строки в datetime object"""
        if isinstance(v, str):
            return datetime.datetime.strptime(v, "%Y-%m-%d %H:%M:%S")
        return v

    async def refresh(self):
        """Обновление информации о выплате"""
        payout = await self._client.get_payout_status(id=self.id)
        self.__dict__ = payout.__dict__



from typing import ClassVar
from cardlink.types.base import BaseCardLinkTypes


class BillTypes(BaseCardLinkTypes):
    """
    Тип счета
    """

    normal: ClassVar[str] = 'normal'
    multi: ClassVar[str] = 'multi'



import datetime
from cardlink.types import SearchedBill
from cardlink.methods.base import CardLinkBaseMethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    import cardlink


class searchBill:
    """bill/search method"""
    class searchBillMethod(CardLinkBaseMethod):
        start_date: datetime.datetime | None = None
        finish_date: datetime.datetime | None = None
        shop_id: str | None = None
        per_page: int | None = None
        cursor: str | None = None

        __return_type__ = SearchedBill
        __api_method__ = "bill/search"
        __request_type__ = "GET"

    async def search_bill(
            self: "cardlink.CardLink",
            start_date: datetime.datetime | None = None,
            finish_date: datetime.datetime | None = None,
            shop_id: str | None = None,
            per_page: int | None = None,
            cursor: str | None = None,
    ) -> SearchedBill:
        """
        Найти счета на оплату
        Поиск по счетам. Можно указать периоды времени или магазин, чтобы найти нужные счета.

        :param start_date: *Optional*. Начальная дата и время для получения счетов в UTC
        :param finish_date: *Optional*. Конечная дата и время для получения счетов в UTC
        :param shop_id: *Optional*. Уникальный идентификатор магазина
        :param per_page: *Optional*. Количество элементов на странице
        :param cursor: *Optional*. Указатель на страницу

        :return: :class:`SearchedBill` object
        """
        params = locals()

        if start_date:
            params["start_date"] = start_date.strftime("%Y-%m-%d %H:%M:%S")
        if finish_date:
            params["finish_date"] = finish_date.strftime("%Y-%m-%d %H:%M:%S")
        if not shop_id:
            params["shop_id"] = self._shop_id

        return await self(self.searchBillMethod(**params))



from cardlink.types import Payouts, Crypto
from cardlink.types.Currency import Currency
from cardlink.methods.base import CardLinkBaseMethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    import cardlink


class createRegularPayoutCreditCard:
    """payout/regular/create method (Credit Card)"""
    class createRegularPayoutCreditCardMethod(CardLinkBaseMethod):
        amount: float | int
        currency: Currency | str
        account_identifier: str
        card_holder: str
        account_currency: Currency | str = 'RUB'
        recipient_pays_commission: bool | None = None
        order_id: str | None = None

        __return_type__ = Payouts
        __api_method__ = "payout/regular/create"
        __request_type__ = "POST"

    async def create_payout_credit_card(
            self: "cardlink.CardLink",
            amount: float | int,
            currency: Currency | str,
            account_identifier: str,
            card_holder: str,
            account_currency: Currency | str | None = None,
            recipient_pays_commission: bool | None = None,
            order_id: str | None = None
    ) -> Payouts:
        """
        Отправить средства на указанные реквизиты (Банковская карта)
        Важно: API предоставляется по запросу через службу поддержки.

        Для того чтобы вывести средства с баланса системы необходимо создать выплату. Запрос на выплату может разбиться на несколько выплат в зависимости от используемого аккаунта. В этом случае, в ответе вы получите список с несколькими выплатами.

        :param amount: Сумма выплаты
        :param currency: Валюта
        :param account_identifier: Идентификатор аккаунта: номер карты
        :param card_holder: Держатель карты. Как указано на карте
        :param account_currency: *Options*. Валюта баланса
        :param recipient_pays_commission: *Optional*.
        :param order_id: *Optional*.
        :return: :class:`cardlink.types.Payout` object
        """

        return await self(self.createRegularPayoutCreditCardMethod(**locals()))


class createPersonalPayoutSBP:
    """payout/regular/create method (SBP)"""
    class createPersonalPayoutSBPMethod(CardLinkBaseMethod):
        amount: float | int
        currency: Currency | str
        account_identifier: str
        account_bank: str
        account_currency: Currency | str = 'RUB'
        recipient_pays_commission: bool | None = None
        order_id: str | None = None

        __return_type__ = Payouts
        __api_method__ = "payout/regular/create"
        __request_type__ = "POST"

    async def create_payout_sbp(
            self: "cardlink.CardLink",
            amount: float | int,
            currency: Currency | str,
            account_identifier: str,
            account_bank: str,
            account_currency: Currency | str = 'RUB',
            recipient_pays_commission: bool | None = None,
            order_id: str | None = None
    ) -> Payouts:
        """
        Отправить средства на указанные реквизиты (СБП)
        Важно: API предоставляется по запросу через службу поддержки.

        Для того чтобы вывести средства с баланса системы необходимо создать выплату. Запрос на выплату может разбиться на несколько выплат в зависимости от используемого аккаунта. В этом случае, в ответе вы получите список с несколькими выплатами.

        :param amount: Сумма выплаты
        :param currency: Валюта
        :param account_identifier: Идентификатор аккаунта: номер карты
        :param account_bank: Member ID банка для account_type=sbp.
        :param account_currency: *Options*. Валюта баланса
        :param recipient_pays_commission: *Optional*.
        :param order_id: *Optional*.
        :return: :class:`cardlink.types.Payout` object
        """

        return await self(self.createPersonalPayoutSBPMethod(**locals()))


class createPersonalPayoutCrypto:
    """/payout/regular/create method (Crypto)"""
    class createPersonalPayoutCryptoMethod(CardLinkBaseMethod):
        amount: float | int
        currency: Currency | str
        account_identifier: str
        account_network: Crypto | str
        account_currency: Currency | str = 'RUB'
        recipient_pays_commission: bool | None = None
        order_id: str | None = None

        __return_type__ = Payouts
        __api_method__ = "payout/regular/create"
        __request_type__ = "POST"

    async def create_payout_crypto(
            self: "cardlink.CardLink",
            amount: float | int,
            currency: Currency | str,
            account_identifier: str,
            account_network: Crypto | str,
            account_currency: Currency | str = 'RUB',
            recipient_pays_commission: bool | None = None,
            order_id: str | None = None
    ) -> Payouts:
        """
        Отправить средства на указанные реквизиты (Криптовалюта)
        Важно: API предоставляется по запросу через службу поддержки.

        Для того чтобы вывести средства с баланса системы необходимо создать выплату. Запрос на выплату может разбиться на несколько выплат в зависимости от используемого аккаунта. В этом случае, в ответе вы получите список с несколькими выплатами.

        :param amount: Сумма выплаты
        :param currency: Валюта
        :param account_identifier: Идентификатор аккаунта: номер карты
        :param account_network: Сеть для отправки криптовалюты/токенов.
        :param account_currency: *Options*. Валюта баланса
        :param recipient_pays_commission: *Optional*.
        :param order_id: *Optional*.
        :return: :class:`cardlink.types.Payout` object
        """


        return await self(self.createPersonalPayoutCryptoMethod(**locals()))


class createPersonalPayoutSteam:
    """payout/regular/create method (Steam)"""
    class createPersonalPayoutSteamMethod(CardLinkBaseMethod):
        amount: float | int
        currency: Currency | str
        account_identifier: str
        account_currency: Currency | str = 'RUB'
        recipient_pays_commission: bool | None = None
        order_id: str | None = None

        __return_type__ = Payouts
        __api_method__ = "payout/regular/create"
        __request_type__ = "POST"

    async def create_payout_steam(
            self: "cardlink.CardLink",
            amount: float | int,
            currency: Currency | str,
            account_identifier: str,
            account_currency: Currency | str = 'RUB',
            recipient_pays_commission: bool | None = None,
            order_id: str | None = None
    ) -> Payouts:
        """
        Отправить средства на указанные реквизиты (Steam)
        Важно: API предоставляется по запросу через службу поддержки.

        Для того чтобы вывести средства с баланса системы необходимо создать выплату. Запрос на выплату может разбиться на несколько выплат в зависимости от используемого аккаунта. В этом случае, в ответе вы получите список с несколькими выплатами.

        :param amount: Сумма выплаты
        :param currency: Валюта
        :param account_identifier: Идентификатор аккаунта: номер карты
        :param account_currency: *Options*. Валюта баланса
        :param recipient_pays_commission: *Optional*.
        :param order_id: *Optional*.

        :return: :class:`Payout` object
        """


        return await self(self.createPersonalPayoutSteamMethod(**locals()))



from typing import ClassVar
from cardlink.types.base import BaseCardLinkTypes


class PayoutStatus(BaseCardLinkTypes):
    """Статус выплаты"""

    new: ClassVar[str] = "NEW"
    moderating: ClassVar[str] = "MODERATING"
    process: ClassVar[str] = "PROCESS"
    fail: ClassVar[str] = "FAIL"
    error: ClassVar[str] = "ERROR"
    success: ClassVar[str] = "SUCCESS"
    declined: ClassVar[str] = "DECLINED"



from cardlink.types import SearchedPayments
from cardlink.methods.base import CardLinkBaseMethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    import cardlink


class billPayments:
    """bill/payments method"""
    class billPaymentsMethod(CardLinkBaseMethod):
        id: str
        per_page: int | None = None
        cursor: str | None = None

        __return_type__ = SearchedPayments
        __api_method__ = "bill/payments"
        __request_type__ = "GET"

    async def bill_payments(
            self: "cardlink.CardLink",
            id: str,
            per_page: int | None = None,
            cursor: str | None = None
    ) -> SearchedPayments:
        """
        Получить платежи, относящиеся к счету на оплату
        Используйте этот метод для получения всех платежей по счету.

        :param id: Уникальный идентификатор счета
        :param per_page: *Optional*. Количество элементов на странице
        :param cursor: *Optional*. Указатель на страницу

        :return: :class:`SearchedPayments` object
        """

        return await self(self.billPaymentsMethod(**locals()))



import datetime
from typing import Optional, Literal
from cardlink.types import Refund, Balances, Bill, SearchedBill, SearchedPayout, Currency, Banks
from cardlink.methods import CardLinkBaseMethod
from cardlink.types import SearchedPayments
from cardlink.types.Payout import Payout
from cardlink.types.SearchedRefund import SearchedRefund
from cardlink.types.base import BaseCardLinkTypes
from cardlink.api.aiohttp import AiohttpSession
from cardlink.error import APIError


class CardLink:
    _token: str
    _shop_id: str
    _session: AiohttpSession

    def __init__(self, token: str, shop_id: str) -> None: ...
    def __call__(self, method: CardLinkBaseMethod) -> BaseCardLinkTypes: ...
    def __auth(self) -> None | APIError: ...

    def create_bill(
        self,
        amount: float | int,
        order_id: Optional[str] = ...,
        description: Optional[str] = ...,
        type: Literal['normal', 'multi'] = ...,
        currency_in: Literal['RUB', 'USD', 'EUR'] = ...,
        custom: Optional[str] = ...,
        payer_pays_commission: Literal[0, 1] = ...,
        payer_email: Optional[str] = ...,
        name: Optional[str] = ...,
        ttl: Optional[int] = ...,
        success_url: Optional[str] = ...,
        fail_url: Optional[str] = ...,
        payment_method: Literal["BANK_CARD", "SBP"] = ...,
        request_fields_email: bool = ...,
        request_fields_phone: bool = ...,
        request_fields_name: bool = ...,
        request_fields_comment: bool = ...,
        items: Optional[list] = ...,
    ) -> Bill | APIError: ...
    def get_bill(
        self,
        id: str,
    ) -> Bill | APIError: ...
    def toggle_activity(
        self,
        id: str,
        active: bool
    ) -> Bill | APIError: ...
    def search_bill(
        self,
        start_date: Optional[datetime.datetime] = ...,
        finish_date: Optional[datetime.datetime] = ...,
        shop_id: Optional[str] = ...,
        per_page: Optional[int] = ...,
        cursor: Optional[str] = ...,
    ) -> SearchedBill: ...
    def bill_payments(
            self,
            id: str,
            per_page: Optional[int] = ...,
            cursor: Optional[str] = ...,
    ) -> SearchedPayments: ...
    def refund_full(
        self,
        payment_id: str = ...
    ) -> Refund | APIError: ...
    def refund_partial(
        self,
        payment_id: str = ...,
        amount: int | float = ...,
    ) -> Refund | APIError: ...
    def get_refund(
            self,
            id: str
    ) -> Refund: ...
    def search_refund(
            self,
            payment_id: str = ...,
            start_date: datetime.datetime = ...,
            finish_date: datetime.datetime = ...,
            per_page: int = ...,
            cursor: str = ...
    ) -> SearchedRefund: ...
    def personal_payout(
        self,
        amount: float | int,
        payout_account_id: str,
        account_currency: Optional[Literal['USD', 'RUB', 'EUR']] = ...,
        recipient_pays_commission: Optional[bool] = ...,
        order_id: Optional[str] = ...,
    ) -> Payout | APIError: ...
    def create_payout_credit_card(
        self,
        amount: float | int,
        currency: Literal['USD', 'RUB', 'EUR'],
        account_identifier: str,
        card_holder: str,
        account_currency: Literal['USD', 'RUB', 'EUR'] = ...,
        recipient_pays_commission: Optional[bool] = ...,
        order_id: Optional[str] = ...,
    ) -> Payout | APIError: ...
    def create_payout_sbp(
        self,
        amount: float | int,
        currency: Literal['USD', 'RUB', 'EUR'],
        account_identifier: str,
        account_bank: str,
        account_currency: Literal['USD', 'RUB', 'EUR'] = ...,
        recipient_pays_commission: Optional[bool] = ...,
        order_id: Optional[str] = ...,
    ) -> Payout | APIError: ...
    def create_payout_crypto(
        self,
        amount: float | int,
        currency: Literal['USD', 'RUB', 'EUR'],
        account_identifier: str,
        account_network: Literal['TRX', 'ETH'],
        account_currency: Literal['USD', 'RUB', 'EUR'] = ...,
        recipient_pays_commission: Optional[bool] = ...,
        order_id: Optional[str] = ...,
    ) -> Payout | APIError: ...
    def create_payout_steam(
        self,
        amount: float | int,
        currency: Currency | str,
        account_identifier: str,
        account_currency: Currency | str = ...,
        recipient_pays_commission: Optional[bool] = ...,
        order_id: Optional[str] = ...,
    ) -> Payout | APIError: ...
    def get_payout(
        self,
        id: Optional[str] = ...,
        order_id: Optional[str] = ...,
    ) -> Payout | APIError: ...
    def search_payout(
        self,
        start_date: Optional[datetime.datetime] = ...,
        finish_date: Optional[datetime.datetime] = ...,
        per_page: Optional[int] = ...,
        cursor: Optional[str] = ...,
    ) -> SearchedPayout: ...
    def get_balance(self) -> Balances | APIError: ...
    def get_banks(self) -> Banks:...
    def search_payments(
        self,
        start_date: Optional[datetime.datetime] = ...,
        finish_date: Optional[datetime.datetime] = ...,
        shop_id: Optional[str] = ...,
        per_page: Optional[int] = ...,
        cursor: Optional[str] = ...,
    ) -> SearchedPayments: ...
    def get_payment(
        self,
        id: Optional[str] = ...,
        refunds: Optional[bool] = ...,
        chargeback: Optional[bool] = ...,
    ) -> Bill: ...



from cardlink.types.Refund import Refund
from cardlink.methods.base import CardLinkBaseMethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    import cardlink


class refundStatus:
    """refund/status method"""
    class refundStatusMethod(CardLinkBaseMethod):
        id: str

        __return_type__ = Refund
        __api_method__ = "refund/status"
        __request_type__ = "GET"

    async def get_refund(
            self: "cardlink.CardLink",
            id: str,
    ) -> Refund:
        """
        Получить статус возврата

        :param id: Уникальный идентификатор возврата

        :return: :class:`Refund` object
        """

        return await self(self.refundStatusMethod(**locals()))



from cardlink.types import Banks
from cardlink.methods.base import CardLinkBaseMethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    import cardlink


class getBanks:
    """payout/dictionaries/sbp_banks method"""
    class getBanksMethod(CardLinkBaseMethod):
        __return_type__ = Banks
        __api_method__ = "payout/dictionaries/sbp_banks"
        __request_type__ = "GET"

    async def get_banks(
            self: "cardlink.CardLink"
    ) -> Banks:
        """
        Получить список банков, доступных для СБП выплат

        :return: :class:`Banks` object
        """

        return await self(self.getBanksMethod(**locals()))



from cardlink.types import Refund
from cardlink.methods.base import CardLinkBaseMethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    import cardlink


class createPartialRefund:
    """refund/partial/create method"""
    class createPartialRefundMethod(CardLinkBaseMethod):
        payment_id: str
        amount: int | float
        __api_method__ = "refund/partial/create"
        __return_type__ = Refund
        __request_type__ = "POST"


    async def refund_partial(
        self: "cardlink.CardLink",
        payment_id: str,
        amount: int | float
    ) -> Refund:
        """
        Сделать частичный возврат средств
        Важно: API предоставляется по запросу через службу поддержки.

        :param payment_id: Уникальный идентификатор платежа
        :param amount: Сумма частичного возврата

        :return: :class:`Refund` object
        """
        return await self(self.createFullRefundMethod(**locals()))



class APIError(Exception):
    def __init__(self, message: str, code: int | None = None):
        super().__init__(message)
        self.code = code
        self.message = message

    def __str__(self):
        if self.code is not None:
            return f"APIError {self.code}: {self.message}"
        return self.message



from cardlink.types.Payout import Payout
from cardlink.types.Meta import Meta
from cardlink.types.Links import Links
from cardlink.types.base import BaseCardLinkTypes


class SearchedPayout(BaseCardLinkTypes):
    """
    Ответ на запрос "search_payout"
    """

    data: list[Payout]
    """Массив выплат"""
    links: Links
    """Ссылки для пагинации"""
    meta: Meta
    """Мета данные пагинации"""



from cardlink.types.base import BaseCardLinkTypes


class Links(BaseCardLinkTypes):
    """
    Ссылки для пагинации
    """

    prev: str | None
    """Ссылка на предыдущую страницу"""
    next: str | None
    """Ссылка на следующую страницу"""



from cardlink.types import Payout
from cardlink.types.base import BaseCardLinkTypes


class Payouts(BaseCardLinkTypes):
    """
    Payouts object.
    """
    data: Payout | list[Payout]
    """Список выводов / вывод"""



from typing import ClassVar
from cardlink.types.base import BaseCardLinkTypes


class BillStatus(BaseCardLinkTypes):
    """
    Статусы счета
    """

    new: ClassVar[str] = "NEW"
    process: ClassVar[str] = "PROCESS"
    underpaid: ClassVar[str] = "UNDERPAID"
    success: ClassVar[str] = "SUCCESS"
    overpaid: ClassVar[str] = "OVERPAID"
    fail: ClassVar[str] = "FAIL"



from abc import ABC
from typing import ClassVar, TypeVar, Type, Generic
from pydantic import BaseModel, ConfigDict


T = TypeVar("T")

class CardLinkBaseMethod(BaseModel, ABC, Generic[T]):
    """Base `CardLink API` method class."""

    model_config = ConfigDict(
        extra="ignore",
        frozen=True,
    )

    __return_type__: Type[T]
    __api_method__: ClassVar[str]
    __request_type__: ClassVar[str]



from cardlink.types import Refund
from cardlink.methods.base import CardLinkBaseMethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    import cardlink


class createFullRefund:
    """refund/full/create method"""
    class createFullRefundMethod(CardLinkBaseMethod):
        payment_id: str
        __api_method__ = "refund/full/create"
        __return_type__ = Refund
        __request_type__ = "POST"

    async def refund_full(
            self: "cardlink.CardLink",
            payment_id: str
    ) -> Refund:
        """
        Сделать полный возврат средств
        Важно: API предоставляется по запросу через службу поддержки.

        :param payment_id: Уникальный идентификатор платежа

        :return: :class:`Refund` object
        """

        return await self(self.createFullRefundMethod(**locals()))



from cardlink.types.base import BaseCardLinkTypes


class CreatedBill(BaseCardLinkTypes):
    """
    Ответ сервера после запроса на создание счета
    """

    link_url: str
    """Ссылка на страницу с QR кодом"""
    link_page_url: str
    """Ссылка на оплату"""
    bill_id: str
    """Уникальный идентификатор счета"""



import datetime

from pydantic import AnyUrl, field_validator
from cardlink.types.BillTypes import BillTypes
from cardlink.types.Currency import Currency
from cardlink.types.base import BaseCardLinkTypes
from cardlink.types.BillStatus import BillStatus


class Bill(BaseCardLinkTypes):
    """
    Bill object

    Source: https://cardlink.link/merchant/api#bill-resource
    """

    id: str
    """Уникальный идентификатор счета"""
    order_id: str | None = None
    """Уникальный идентификатор заказа на вашей стороне"""
    active: bool | None = None
    """Флаг активности счета"""
    status: BillStatus | str | None = None
    """Статус счета"""
    amount: int | float | None = None
    """Сумма, на которую выставлен счет"""
    type: BillTypes | str | None = None
    """Тип счета"""
    created_at: datetime.datetime | str | None = None
    """Дата и время создания счета"""
    currency_in: Currency | str | None = None
    """Валюта, в которой оплачивается счет"""
    ttl: int | None = None
    """Время жизни счета на оплату в секундах"""
    link_url: AnyUrl | str | None = None
    """Ссылка на страницу с QR кодом"""
    link_page_url: AnyUrl | str | None = None
    """Ссылка на оплату"""

    @field_validator("created_at", mode="before")
    @classmethod
    def parse_datetime(cls, v):
        """Преобразование строки в datetime object"""
        if isinstance(v, str):
            return datetime.datetime.strptime(v, "%Y-%m-%d %H:%M:%S")
        return v

    async def toggle_active(self):
        """Активация/деактивация счета на оплату"""
        bill = await self._client.toggle_activity(id=self.id, active=(not self.active))
        self.__dict__ = bill.__dict__

    async def refresh(self):
        """Обновление информации о счете"""
        bill = await self._client.get_bill_status(id=self.id)
        self.__dict__ = bill.__dict__



from cardlink.types.base import BaseCardLinkTypes
from cardlink.types.Balance import Balance


class Balances(BaseCardLinkTypes):
    """
    Balances object
    """

    balances: list[Balance]
    """Массив, содержащий информацию о балансах мерчанта"""



import datetime
from typing import Any

from pydantic import field_validator

from cardlink.types.base import BaseCardLinkTypes
from cardlink.types.PaymentStatus import PaymentStatus
from cardlink.types.Currency import Currency


class Payment(BaseCardLinkTypes):
    """
    Payment object

    Source: https://cardlink.link/merchant/api#chargeback-resource
    """

    id: str | None = None
    """Уникальный идентификатор платежа"""
    bill_id: str | None = None
    """Уникальный идентификатор счета, которому принадлежит платеж"""
    status: PaymentStatus | str | None = None
    """Статус платежа"""
    amount: int | float | None = None
    """Сумма платежа"""
    commission: int | float | None = None
    """Комиссия"""
    account_amount: int | float | None = None
    """Сумма зачисления на баланс"""
    account_currency_code: Currency | str | None = None
    """Валюта зачисления на баланс"""
    refunded_amount: int |float  | None = None
    """Возвращенная сумма"""
    from_card: str | None = None
    """Номер карты, с которой произошла оплата"""
    currency_in: Currency | str | None = None
    """Валюта, в которой оплачивается счет"""
    created_at: datetime.datetime | None = None
    """Дата и время создания платежа"""

    @field_validator("created_at", mode="before")
    @classmethod
    def parse_datetime(cls, v):
        """Преобразование строки в datetime object"""
        if isinstance(v, str):
            return datetime.datetime.strptime(v, "%Y-%m-%d %H:%M:%S")
        return v

    async def refresh(self):
        """Обновление информации о платеже"""
        payment = await self._client.get_payment_status(id=self.id)
        self.__dict__ = payment.__dict__



from cardlink.types.base import BaseCardLinkTypes


class Meta(BaseCardLinkTypes):
    """
    Мета данные пагинации
    """

    path: str
    """Ссылка на страницу без курсора"""
    per_page: int
    """Количество элементов на странице"""
    prev_cursor: str | None = None
    """Указатель на предыдущую страницу"""
    next_cursor: str | None = None
    """Указатель на следующую страницу"""



from cardlink.types import Bill
from cardlink.methods.base import CardLinkBaseMethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    import cardlink


class billStatus:
    """bill/status method"""
    class billStatusMethod(CardLinkBaseMethod):
        id: str

        __return_type__ = Bill
        __api_method__ = "bill/status"
        __request_type__ = "GET"

    async def get_bill(
            self: "cardlink.CardLink",
            id: str
    ) -> Bill:
        """
        Получить статус счета на оплату
        Если счет имеет тип MULTI, то его статус не будет изменяться.

        :param id: Уникальный идентификатор счета

        :return: :class:`Bill` object
        """

        return await self(self.billStatusMethod(**locals()))



from typing import ClassVar
from cardlink.types.base import BaseCardLinkTypes


class Currency(BaseCardLinkTypes):
    """Доступные валюты"""

    rub: ClassVar[str] = "RUB"
    usd: ClassVar[str] = "USD"
    eur: ClassVar[str] = "EUR"



from typing import Literal, TYPE_CHECKING
from cardlink.types import Item, Bill, Currency, CreatedBill, PaymentMethod
from cardlink.types.BillTypes import BillTypes
from cardlink.methods.base import CardLinkBaseMethod

if TYPE_CHECKING:
    import cardlink


class createBill:
    """bill/create method"""
    class createBillMethod(CardLinkBaseMethod):
        amount: float | int
        shop_id: str | None = None
        order_id: str | None = None
        description: str | None = None
        type: Literal['normal', 'multi'] = 'normal'
        currency_in: Literal['RUB', 'USD', 'EUR'] = 'RUB'
        custom: str | None = None
        payer_pays_commission: bool | None = None
        payer_email: str | None = None
        name: str | None = None
        ttl: int | None = None
        success_url: str | None = None
        fail_url: str | None = None
        payment_method: Literal["BANK_CARD", "SBP"] = "SBP"
        request_fields_email: bool = False
        request_fields_phone: bool = False
        request_fields_name: bool = False
        request_fields_comment: bool = False
        items: list[Item] | None = None

        __api_method__ = "bill/create"
        __return_type__ = CreatedBill
        __request_type__ = "POST"

        def model_dump(self, *args, **kwargs):
            data = super().model_dump(*args, **kwargs)

            data["request_fields"] = {
                "email": data.pop("request_fields_email", None),
                "phone": data.pop("request_fields_phone", None),
                "name": data.pop("request_fields_name", None),
                "comment": data.pop("request_fields_comment", None),
            }

            return data

    async def create_bill(
            self: "cardlink.CardLink",
            amount: float | int,
            order_id: str | None = None,
            description: str | None = None,
            type: BillTypes | str = 'normal',
            currency_in: Currency | str = 'RUB',
            custom: str | None = None,
            payer_pays_commission: bool | None = None,
            payer_email: str | None = None,
            name: str | None = None,
            ttl: int | None = None,
            success_url: str | None = None,
            fail_url: str | None = None,
            payment_method: PaymentMethod | str = "SBP",
            request_fields_email: bool = False,
            request_fields_phone: bool = False,
            request_fields_name: bool = False,
            request_fields_comment: bool = False,
            items: list[Item] | None = None
    ) -> Bill:
        """
        Создать счет на оплату
        Для того чтобы провести оплату необходимо создать счет. Если тип счета многоразовый, то по одному счету можно произвести несколько платежей. Домен в shop_url должен совпадать с доментом success_url и fail_url.

        :param amount: Сумма счета на оплату
        :param order_id: *Optional*. Уникальный идентификатор заказа. Будет возвращен в postback
        :param description: *Optional*. Описание платежа
        :param type: *Optional*. Тип платежа. Одноразовый или многоразовый. Если выбран одноразовый, то второй раз оплатить не получится
        :param currency_in: *Optional*. Валюта, в которой оплачивается счет. Если не передана, то используется валюта магазина. Если shop_id не определен, то используется RUB
        :param custom: *Optional*. Произвольное поле. Будет возвращено в postback
        :param payer_pays_commission: *Optional*. Параметр, который указывает на то, кто будет оплачивать комиссию за входящий платёж
        :param payer_email: *Optional*. Параметр, который заполняет email клиента на платёжной странице
        :param name: *Optional*. Название ссылки. Укажите, за что принимаете средства. Этот текст будет отображен в платежной форме
        :param ttl: *Optional*. Время жизни счета на оплату в секундах
        :param success_url: *Optional*. Страница успешной оплаты
        :param fail_url: *Optional*. Страница неуспешной оплаты
        :param payment_method: *Optional*. Способ оплаты. Если указан этот параметр, то при переходе на платежную форму этот способ оплаты будет выбран автоматически, без возможности выбора другого способ оплаты
        :param request_fields_email: *Optional*. Обязательный запрос электронной почты у плательщика
        :param request_fields_phone: *Optional*. Обязательный запрос номера телефона у плательщика
        :param request_fields_name: *Optional*. Обязательный запрос ФИО у плательщика
        :param request_fields_comment: *Optional*. Обязательный запрос комментария у плательщика
        :param items: *Optional*. Список товаров

        :return: :class:`Bill` object
        """

        params = locals()
        if params['payer_pays_commission']:
            params['payer_pays_commission'] = int(params['payer_pays_commission'])

        params['shop_id'] = self._shop_id

        created_bill: CreatedBill = await self(self.createBillMethod(**params))

        bill: Bill = await self.get_bill_status(id=created_bill.bill_id)
        bill.link_url = created_bill.link_url
        bill.link_page_url = created_bill.link_page_url

        return bill



from abc import ABC
from cardlink.error import APIError
from cardlink.types import Payout


class BaseSession(ABC):
    """
    Абстрактный класс.
    """

    def _check_response(self, return_type, code, data, api_method, client):
        if data.get('success') or data.get('success') == 'true':
            data.pop('success')

            if api_method in [
                'payout/dictionaries/sbp_banks',
                'payout/personal/create',
                'payout/regular/create'
            ]:
                data = data['data']
                if isinstance(data, list):
                    return Payout.model_validate(data, context={"client": client})

            return return_type.model_validate(data, context={"client": client})
        raise APIError(message=data.get('message'), code=code)



from cardlink.types.base import BaseCardLinkTypes
from cardlink.types.Bill import Bill
from cardlink.types.Meta import Meta
from cardlink.types.Links import Links


class SearchedBill(BaseCardLinkTypes):
    """
    Ответ на запрос "search_bill"
    """

    data: list[Bill]
    """Массив счетов она оплату, удовлетворяющих криетриям запроса"""
    links: Links
    """Ссылки для пагинации"""
    meta: Meta
    """Мета данные пагинации"""



from cardlink.types.base import BaseCardLinkTypes


class Item(BaseCardLinkTypes):
    """
    Список товаров.
    """

    name: str
    price: str
    quantity: str
    category: str
    extra_phone: str



import datetime
from cardlink.types import SearchedRefund
from cardlink.methods.base import CardLinkBaseMethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    import cardlink


class searchRefund:
    """refund/search method"""
    class searchRefundMethod(CardLinkBaseMethod):
        payment_id: str | None = None
        start_date: datetime.datetime | None = None
        finish_date: datetime.datetime | None = None
        per_page: int | None = None
        cursor: str | None = None

        __return_type__ = SearchedRefund
        __api_method__ = "refund/search"
        __request_type__ = "GET"

    async def search_refund(
            self: "cardlink.CardLink",
            payment_id: str | None = None,
            start_date: datetime.datetime | None = None,
            finish_date: datetime.datetime | None = None,
            per_page: int | None = None,
            cursor: str | None = None,
    ) -> SearchedRefund:
        """
        Получить возвраты

        :param payment_id: *Optional*. ID платежа
        :param start_date: *Optional*. Начальная дата и время для получения возвратов в UTC
        :param finish_date: *Optional*. Конечная дата и время для получения возвратов в UTC
        :param per_page: *Optional*. Количество элементов на странице
        :param cursor: *Optional*. Указатель на страницу

        :return: :class:`SearchedRefund` object
        """
        params = locals()

        if start_date:
            params["start_date"] = start_date.strftime("%Y-%m-%d %H:%M:%S")
        if finish_date:
            params["finish_date"] = finish_date.strftime("%Y-%m-%d %H:%M:%S")

        return await self(self.searchRefundMethod(**params))



import datetime
from pydantic import field_validator
from cardlink.types.RefundStatus import RefundStatus
from cardlink.types.Currency import Currency
from cardlink.types.RefundType import RefundType
from cardlink.types.base import BaseCardLinkTypes


class Refund(BaseCardLinkTypes):
    """
    Refund object

    Source: https://cardlink.link/merchant/api#refund-resource
    """

    id: str | None = None
    """Уникальный идентификатор возврата"""
    status: RefundStatus | str | None = None
    """Статус возврата"""
    amount: int | float | None = None
    """Сумма возврата"""
    currency_in: Currency | str | None = None
    """Валюта"""
    entity_type: str | RefundType | None = None
    """Тип возврата"""
    entity_id: str | None = None
    """Уникальный идентификатор платежа, по которому производится возврат"""
    created_at: datetime.datetime | None = None
    """Дата и время создания возврата"""

    @field_validator("created_at", mode="before")
    @classmethod
    def parse_datetime(cls, v):
        """Преобразование строки в datetime object"""
        if isinstance(v, str):
            return datetime.datetime.strptime(v, "%Y-%m-%d %H:%M:%S")
        return v

    async def refresh(self):
        """Обновление информации о возврате"""
        refund = await self._client.get_refund_status(id=self.id)
        self.__dict__ = refund.__dict__



from cardlink.types.base import BaseCardLinkTypes               # noqa: F401
from cardlink.types.Balance import Balance                      # noqa: F401
from cardlink.types.Balances import Balances                    # noqa: F401
from cardlink.types.Bank import Bank                            # noqa: F401
from cardlink.types.Banks import Banks                          # noqa: F401
from cardlink.types.Bill import Bill                            # noqa: F401
from cardlink.types.BillStatus import BillStatus                # noqa: F401
from cardlink.types.BillTypes import BillTypes                  # noqa: F401
from cardlink.types.CreatedBill import CreatedBill              # noqa: F401
from cardlink.types.Crypto import Crypto                        # noqa: F401
from cardlink.types.Currency import Currency                    # noqa: F401
from cardlink.types.Item import Item                            # noqa: F401
from cardlink.types.Links import Links                          # noqa: F401
from cardlink.types.Meta import Meta                            # noqa: F401
from cardlink.types.Payment import Payment                      # noqa: F401
from cardlink.types.PaymentMethod import PaymentMethod          # noqa: F401
from cardlink.types.PaymentStatus import PaymentStatus          # noqa: F401
from cardlink.types.Payout import Payout                        # noqa: F401
from cardlink.types.Payouts import Payouts                      # noqa: F401
from cardlink.types.PayoutStatus import PayoutStatus            # noqa: F401
from cardlink.types.Refund import Refund                        # noqa: F401
from cardlink.types.RefundStatus import RefundStatus            # noqa: F401
from cardlink.types.RefundType import RefundType                # noqa: F401
from cardlink.types.SearchedBill import SearchedBill            # noqa: F401
from cardlink.types.SearchedPayments import SearchedPayments    # noqa: F401
from cardlink.types.SearchedPayout import SearchedPayout        # noqa: F401
from cardlink.types.SearchedRefund import SearchedRefund        # noqa: F401


__all__ = [
    "Balance",
    "Balances",
    "Bank",
    "Banks",
    "Bill",
    "BillTypes",
    "BillStatus",
    "CreatedBill",
    "Crypto",
    "Currency",
    "Item",
    "Links",
    "Meta",
    "Payment",
    "PaymentMethod",
    "PaymentStatus",
    "Payout",
    "Payouts",
    "PayoutStatus",
    "Refund",
    "RefundStatus",
    "RefundType",
    "SearchedBill",
    "SearchedPayments",
    "SearchedPayout",
    "SearchedRefund",
    "BaseCardLinkTypes",
]



import ssl
from urllib.parse import urlencode
import certifi
from aiohttp import ClientSession, TCPConnector
from cardlink.api.base import BaseSession
from cardlink.utils.clean_dict import clean_dict

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    import cardlink
    import cardlink.methods.base


class AiohttpSession(BaseSession):
    """
    HTTP-сессия на aiohttp
    """

    def __init__(self):
        self._session: ClientSession | None = None

    async def post_request(
            self,
            token,
            method: "cardlink.methods.base.CardLinkBaseMethod",
            client: "cardlink.CardLink"
    ):
        method_data = clean_dict(method.model_dump())

        ssl_context = ssl.create_default_context(cafile=certifi.where())
        self._session = ClientSession(
            connector=TCPConnector(
                ssl_context=ssl_context,
            ),
        )
        try:
            async with self._session.post(
                url=f"https://cardlink.link/api/v1/{method.__api_method__}",
                json=method_data,
                headers={
                    "Authorization": f"Bearer {token}",
                    "Content-Type": "application/json"
                },
            ) as resp:
                response = self._check_response(return_type=method.__return_type__, code=resp.status, data=await resp.json(), api_method=method.__api_method__, client=client)
            return response
        finally:
            await self._session.close()
    async def get_request(
            self,
            token,
            method: "CardLinkBaseMethod",
            client: "CardLink"
    ):
        method_data = clean_dict(method.model_dump())

        ssl_context = ssl.create_default_context(cafile=certifi.where())
        self._session = ClientSession(
            connector=TCPConnector(
                ssl_context=ssl_context,
            ),
        )
        try:
            async with self._session.get(
                url=f"https://cardlink.link/api/v1/{method.__api_method__}?{urlencode(method_data)}",
                headers={
                    "Authorization": f"Bearer {token}",
                    "Content-Type": "application/json"
                },
            ) as resp:
                response = self._check_response(return_type=method.__return_type__, code=resp.status, data=await resp.json(), api_method=method.__api_method__, client=client)
            return response
        finally:
            await self._session.close()




from cardlink.types import Payout
from cardlink.error import APIError
from cardlink.methods.base import CardLinkBaseMethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    import cardlink


class payoutStatus:
    """payout/status method"""
    class payoutStatusMethod(CardLinkBaseMethod):
        id: str | None = None
        order_id: str | None = None

        __return_type__ = Payout
        __api_method__ = "payout/status"
        __request_type__ = "GET"

    async def get_payout(
            self: "cardlink.CardLink",
            id: str | None = None,
            order_id: str | None = None
    ) -> Payout:
        """
        Получить статус выплаты

        :param id: Уникальный идентификатор выплаты. Обязателен, если не передан order_id
        :param order_id: Уникальный идентификатор заказа. Обязателен, если не передан id

        :return::class:`Payout` object
        """
        if id is None and order_id is None:
            return APIError(message="Argument not passed")

        return await self(self.payoutStatusMethod(**locals()))



from cardlink.types import Balances
from cardlink.methods.base import CardLinkBaseMethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    import cardlink


class getBalance:
    """merchant/balance method"""
    class getBalanceMethod(CardLinkBaseMethod):
        __return_type__ = Balances
        __api_method__ = "merchant/balance"
        __request_type__ = "GET"

    async def get_balance(
            self: "cardlink.CardLink"
    ) -> Balances:
        """
        Получить баланс

        :return: :class:`Balances` object
        """

        return await self(self.getBalanceMethod(**locals()))



from typing import ClassVar
from cardlink.types.base import BaseCardLinkTypes


class PaymentMethod(BaseCardLinkTypes):
    """Способы оплаты"""

    bank_card: ClassVar[str] = "BANK_CARD"
    sbp: ClassVar[str] = "SBP"



def clean_dict(data: dict) -> dict:
    cleaned = {}
    for k, v in data.items():
        if isinstance(v, dict):
            nested = clean_dict(v)
            if nested:
                cleaned[k] = nested
        elif v is not None:
            cleaned[k] = v
    return cleaned



from cardlink.types import Bill, Payment
from cardlink.methods.base import CardLinkBaseMethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    import cardlink


class paymentStatus:
    """payment/status method"""
    class paymentStatusMethod(CardLinkBaseMethod):
        id: str
        refunds: bool = True
        chargeback: bool = False

        __return_type__ = Payment
        __api_method__ = "payment/status"
        __request_type__ = "GET"

    async def get_payment(
            self: "cardlink.CardLink",
            id: str,
            refunds: bool | None = None,
            chargeback: bool | None = None
    ) -> Payment:
        """
        Получить статус платежа

        :param id: Уникальный идентификатор платежа
        :param refunds: *Optional*. Осуществлен ли возврата платежа
        :param chargeback: *Optional*. Осуществлен ли чарджбэк платежа

        :return: :class:`Payment` object
        """

        return await self(self.paymentStatusMethod(**locals()))




from cardlink.types.Payment import Payment
from cardlink.types.Meta import Meta
from cardlink.types.Links import Links
from cardlink.types.base import BaseCardLinkTypes


class SearchedPayments(BaseCardLinkTypes):
    """
    Ответ на запрос "search_payment"
    """

    data: list[Payment]
    """Информация о платеже"""
    links: Links
    """Ссылки для пагинации"""
    meta: Meta
    """Мета данные пагинации"""



import ssl
from urllib.error import HTTPError
from urllib.request import Request, urlopen
import certifi
from cardlink.error import APIError


def token_validate(token) -> bool:
    url = f"https://cardlink.link/api/v1/merchant/balance"
    ssl_context = ssl.create_default_context(cafile=certifi.where())

    try:
        request = Request(
            url,
            headers={"Authorization": f"Bearer {token}"}
        )
        with urlopen(request, context=ssl_context) as resp:
            status_code = resp.getcode()
            body = resp.read()
            return True
    except HTTPError as e:
        status_code = e.code
        body = e.read()
        raise APIError(message=body, code=status_code)



from cardlink.api.aiohttp import AiohttpSession
from cardlink.methods import CardLinkBaseMethod
from cardlink.loggers import client
from cardlink.methods import Methods
from cardlink.error import APIError
from cardlink.utils.check_token import token_validate


class CardLink(Methods):
    """
    Client class providing API methods.

    :param token: CardLink API token
    :param shop_id: SHOP ID
    :param _session: HTTP Session
    """

    def __init__(self, token: str, shop_id: str):
        self._token = token
        self._shop_id = shop_id
        self._session = AiohttpSession()
        self.__auth()


    def __auth(self):
        result_auth = token_validate(token=self._token)
        if not result_auth:
            raise APIError("Authorization failed", 401)
        client.info("Successful authorization in CardLink API")

    async def __call__(
            self,
            method: CardLinkBaseMethod
    ):
        """
        Request method.

        Метод дло отправки запросов к API.

        :param method: CardLinkBaseMethod object
        :return: :class:`BaseCardLinkTypes`
        """
        client.debug(
            "Requesting: /%s with payload %s",
            method.__api_method__,
            method.model_dump_json(),
        )
        if method.__request_type__ == "POST":
            return await self._session.post_request(self._token, method, self)
        return await self._session.get_request(self._token, method, self)



    # async def create_invoice(self, *args, **kwargs):
    #     return await create_invoice_method(self, *args, **kwargs)
    # async def create_full_refund(self, *args, **kwargs):
    #     return await create_full_refund_method(self, *args, **kwargs)
    # async def create_partial_refund(self, *args, **kwargs):
    #     return await create_partial_refund_method(self, *args, **kwargs)
    # async def create_personal_payout(self, *args, **kwargs):
    #     return await create_personal_payout_method(self, *args, **kwargs)
    # async def create_payout_credit_card(self, *args, **kwargs):
    #     return await create_payout_credit_card_method(self, *args, **kwargs)
    # async def create_payout_steam(self, *args, **kwargs):
    #     return await create_payout_steam_method(self, *args, **kwargs)
    # async def create_payout_crypto(self, *args, **kwargs):
    #     return await create_payout_crypto_method(self, *args, **kwargs)
    # async def create_payout_sbp(self, *args, **kwargs):
    #     return await create_payout_sbp_method(self, *args, **kwargs)
    # async def get_balance(self):
    #     return await get_balance_method(self)
    # async def get_invoice_status(self, *args, **kwargs):
    #     return await get_invoice_status_method(self, *args, **kwargs)
    # async def toggle_activity(self, *args, **kwargs):
    #     return await toggle_activity_method(self, *args, **kwargs)
    # async def get_payout_status(self, *args, **kwargs):
    #     return await get_payout_status_method(self, *args, **kwargs)
    # async def search_invoice(self, *args, **kwargs):
    #     return await search_invoice_method(self, *args, **kwargs)
    # async def search_payments(self, *args, **kwargs):
    #     return await search_payments_method(self, *args, **kwargs)
    # async def search_payout(self, *args, **kwargs):
    #     return await search_payout_method(self, *args, **kwargs)
    # async def get_payment_status(self, *args, **kwargs):
    #     return await get_payment_status_method(self, *args, **kwargs)






from cardlink.methods.billPayments import billPayments
from cardlink.methods.billStatus import billStatus
from cardlink.methods.createBill import createBill
from cardlink.methods.createFullRefund import createFullRefund
from cardlink.methods.createPartialRefund import createPartialRefund
from cardlink.methods.createPersonalPayout import createPersonalPayout
from cardlink.methods.createRegularPayout import createRegularPayoutCreditCard, createPersonalPayoutSBP, createPersonalPayoutSteam, createPersonalPayoutCrypto
from cardlink.methods.getBalance import getBalance
from cardlink.methods.getBanks import getBanks
from cardlink.methods.paymentStatus import paymentStatus
from cardlink.methods.payoutStatus import payoutStatus
from cardlink.methods.refundStatus import refundStatus
from cardlink.methods.searchBill import searchBill
from cardlink.methods.searchPayments import searchPayments
from cardlink.methods.searchPayout import searchPayout
from cardlink.methods.searchRefund import searchRefund
from cardlink.methods.toggleActivity import toggleActivity
from cardlink.methods.base import CardLinkBaseMethod


class Methods(
    billPayments,
    billStatus,
    createBill,
    createFullRefund,
    createPartialRefund,
    createPersonalPayout,
    createRegularPayoutCreditCard, createPersonalPayoutSBP, createPersonalPayoutSteam, createPersonalPayoutCrypto,
    getBalance,
    getBanks,
    paymentStatus,
    payoutStatus,
    refundStatus,
    searchBill,
    searchPayout,
    searchPayments,
    searchRefund,
    toggleActivity
):
    pass


__all__ = [
    "CardLinkBaseMethod"
]



from cardlink.types.Bank import Bank
from cardlink.types.base import BaseCardLinkTypes


class Banks(BaseCardLinkTypes):
    """
    Banks object
    """

    data: list[Bank]
    """Список банков"""



from typing import ClassVar
from cardlink.types.base import BaseCardLinkTypes


class Crypto(BaseCardLinkTypes):
    """Доступные сети для вывода средств"""

    eth: ClassVar[str] = "ETH"
    trx: ClassVar[str] = "TRX"



from cardlink.types.base import BaseCardLinkTypes


class Bank(BaseCardLinkTypes):
    """
    Bank object

    Source: https://cardlink.link/merchant/api#sbp-bank-resource
    """

    member_id: int
    """Member ID банка в системе СБП"""
    name: str
    """Название банка"""
    name_en: str
    """Название банка на английском языке"""
    bic: int
    """БИК банка"""



from decimal import Decimal

from cardlink.types.base import BaseCardLinkTypes
from cardlink.types.Currency import Currency


class Balance(BaseCardLinkTypes):
    """
    Balance object

    Source: https://cardlink.link/merchant/api#balance-resource
    """

    currency: Currency | str
    """Валюта баланса"""
    balance_available: int
    """Доступный баланс"""
    balance_locked: Decimal
    """Заблокированный баланс во время выплаты средств"""
    balance_hold: Decimal
    """Временно удержанный баланс. Переходит в доступный спустя время"""




from pydantic import BaseModel, PrivateAttr, ConfigDict
from typing import TYPE_CHECKING, Optional

if TYPE_CHECKING:
    import cardlink


class BaseCardLinkTypes(BaseModel):
    """Base object"""

    _client: Optional["cardlink.CardLink"] = PrivateAttr()

    model_config = ConfigDict(
        extra="ignore",
        arbitrary_types_allowed=True,
        populate_by_name=True
    )

    def model_post_init(self, ctx: dict) -> None:
        self._client = ctx.get("client")



from typing import Literal
from cardlink.types import Payouts
from cardlink.methods.base import CardLinkBaseMethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    import cardlink


class createPersonalPayout:
    """payout/personal/create method"""
    class createPersonalPayoutMethod(CardLinkBaseMethod):
        amount: float | int
        payout_account_id: str
        account_currency: Literal['USD', 'RUB', 'EUR'] | None = None
        recipient_pays_commission: bool | None = None
        order_id: str | None = None

        __return_type__ = Payouts
        __api_method__ = "payout/personal/create"
        __request_type__ = "POST"

    async def personal_payout(
            self: "cardlink.CardLink",
            amount: float | int,
            payout_account_id: str,
            account_currency: Literal['USD', 'RUB', 'EUR'] | None = None,
            recipient_pays_commission: bool | None = None,
            order_id: str | None = None
    ) -> Payouts:
        """
        Создать выплату на привязанный платежный аккаунт
        Важно: API предоставляется по запросу через службу поддержки.

        Для того чтобы вывести средства с баланса системы необходимо создать выплату. Запрос на выплату может разбиться на несколько выплат в зависимости от используемого аккаунта. В этом случае, в ответе вы получите список с несколькими выплатами.

        :param amount: Сумма выплаты
        :param payout_account_id: Уникальный идентификатор платежного аккаунта, на который будет произведена выплата
        :param account_currency: *Optional*. Валюта баланса, с которого необходимо списать средства за выплату
        :param recipient_pays_commission: *Optional*. Параметр отвечающий за то, кто платит комиссию (true-комиссию платит получающий выплату, если false-то комиссия будет вычтена с баланса аккаунта)
        :param order_id: *Optional*. Уникальный идентификатор заказа

        :return: :class:`Payout` object
        """
        return await self(self.createPersonalPayoutMethod(**locals()))
